package de.controller;

import static org.springframework.http.HttpHeaders.ACCEPT;
import static org.springframework.http.MediaType.APPLICATION_FORM_URLENCODED_VALUE;

import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import de.domain.SparqlQueryEvaluator;
import lombok.extern.slf4j.Slf4j;


@RestController
@Slf4j
public class SparqlController {

	@Autowired
	private SparqlQueryEvaluator sparqlQueryEvaluator;

	@Autowired
	public SparqlController(SparqlQueryEvaluator sparqlQueryEvaluator) {
		this.sparqlQueryEvaluator = sparqlQueryEvaluator;
	}

	@RequestMapping(value = "/actors/{actorId}/sparql", method = RequestMethod.POST, consumes = APPLICATION_FORM_URLENCODED_VALUE)
	public void sparqlPostURLencoded(@PathVariable String actorId, 
			@RequestParam(value = "default-graph-uri", required = false) String defaultGraphUri,
			@RequestParam(value = "named-graph-uri", required = false) String namedGraphUri,
			@RequestParam(value = "query") String query, @RequestHeader(ACCEPT) String acceptHeader,
			HttpServletRequest request, HttpServletResponse response) throws IOException {
		doSparql(actorId, request, query, acceptHeader, defaultGraphUri, namedGraphUri, response);
	}

	@RequestMapping(value = "/actors/{actorId}/sparql", method = RequestMethod.GET)
	public void sparqlGet(@PathVariable String actorId, @RequestParam(value = "default-graph-uri", required = false) String defaultGraphUri,
			@RequestParam(value = "named-graph-uri", required = false) String namedGraphUri,
			@RequestParam(value = "query") String query, @RequestHeader(ACCEPT) String acceptHeader,
			HttpServletRequest request, HttpServletResponse response) throws IOException {
		doSparql(actorId, request, query, acceptHeader, defaultGraphUri, namedGraphUri, response);
	}

	private void doSparql(String actorId, HttpServletRequest request, String query, String acceptHeader, String defaultGraphUri,
			String namedGraphUri, HttpServletResponse response) throws IOException {
		try {
			sparqlQueryEvaluator.evaluate(actorId, query, acceptHeader, defaultGraphUri, namedGraphUri, response.getOutputStream());
		} catch (RuntimeException re) {
			log.error("error while evaluating query", re);
			response.sendError(HttpServletResponse.SC_BAD_REQUEST);
		} 
	}
}
